@@ -1,6 +1,6 @@
 
 /*
- *	@(#)mail_reader.c	1.27 10/23/94
+ *	@(#)mail_reader.c	1.32 12/16/94
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -43,8 +43,10 @@
 
 static	char	*line;
 static	int	line_length;
+static	int	read_only = FALSE;
 
 extern	char	default_mail_file [];
+extern	FILE	*mail_fp;
 
 #ifdef SYSV
 void	usleep(n)
@@ -203,10 +205,10 @@ MESSAGE	*m;
 char	*s;
 
 {
-	if (!m->message)
-		m->message = new_buffer();
+	if (!m->body)
+		m->body = new_buffer();
 
-	add_to_buffer(m->message,s,strlen(s));
+	add_to_buffer(m->body,s,strlen(s));
 }
 
 /* ret_string is for general use */
@@ -480,7 +482,7 @@ MESSAGE	*m;
 	if (m->decrypted)
 		m_in = (char *) m->decrypted->message;
 	else
-		m_in = (char *)m->message->message;
+		m_in = (char *)message_contents(m)->message;
 
 	while (*m_in && *m_in != '\n') {
 		line[i++] = *m_in++;
@@ -501,6 +503,7 @@ MESSAGE	*m;
 	newm = new_message();
 	add_header(newm,line);
 	add_header(newm,ret_string);
+	set_mem_message (newm);
 
 	if (!no_from) {
 
@@ -615,9 +618,8 @@ MESSAGE	*m;
 		newm->header = b;
 	}
 
-	newm->size = newm->message->length;
+	newm->size = newm->body->length;
 	newm->lines = line_count;
-
 	newm->time_secs = get_time (m);
 
 	return newm;
@@ -627,7 +629,6 @@ MESSAGE	*m;
 #define CONTENT_LENGTH	1
 #define FROM_LINE	2
 
-static	FILE	*mail_fp;
 static	time_t	last_mail_read;
 static	long	last_pos;
 static	char	last_file [MAXPATHLEN];
@@ -641,6 +642,7 @@ int	kill_messages;
 	int	i;
 	int	c;
 	MESSAGE	*m;
+	BUFFER	*mb;
 	static	char	temp[LINE_LENGTH];
 	char	*p,*d;
 	int	killed = FALSE;
@@ -652,6 +654,7 @@ int	kill_messages;
 #endif
 
 	i=0;
+	mb = new_buffer ();
 
 	if (!line) {
 		line_length = LINE_LENGTH;
@@ -683,6 +686,7 @@ int	kill_messages;
 		method = LINE_COUNT;
 
 		m = new_message();
+		clear_buffer (mb);
 
 #ifdef ACCEPT_PATH
 		if (!no_from) {
@@ -782,21 +786,23 @@ int	kill_messages;
 
 		/* If some lines, read them */
 
+		m->offset = ftell (mail_fp);
+
 		if (lines > 0 || method != LINE_COUNT) {
 
 			finished = FALSE;
 
 			if (method == CONTENT_LENGTH) {
-				m->message->message = (byte *)malloc(m->size+2);
+				mb->message = (byte *)malloc(m->size+2);
 
-				fread (m->message->message,
+				fread (mb->message,
 					m->size, 1, fp);
-				m->message->length = m->size;
-				m->message->size = m->size+2;
+				mb->length = m->size;
+				mb->size = m->size+2;
 
-				m->message->message[m->size] = 0;
-				m->lines = count_lines (m->message->message,
-					m->message->length);
+				mb->message[m->size] = 0;
+				m->lines = count_lines (mb->message,
+					mb->length);
 
 				finished = TRUE;
 			}
@@ -830,25 +836,20 @@ int	kill_messages;
 							not_pgp_message = TRUE;
 				}
 				
-				add_body(m,line);
-				add_body(m,ret_string);
+				add_to_buffer(mb,line, strlen (line));
+				add_to_buffer(mb,ret_string,
+					strlen (ret_string));
 
 				if (method == LINE_COUNT) {
 					finished = (--lines <= 0);
 				}
 				else if (method == CONTENT_LENGTH) {
-					finished = (m->message->length >= 
+					finished = (mb->length >= 
 						m->size);
 				}
 			}
 
-			m->size = m->message->length;
-		}
-		else {
-			if (!m->message) {
-				m->message = new_buffer();
-				m->size = 0;
-			}
+			m->size = mb->length;
 		}
 
 		if (!killed) {
@@ -858,6 +859,8 @@ int	kill_messages;
 			add_to_deleted (m);
 		}
 
+		set_file_message (m);
+
 		m->time_secs = get_time (m);
 
 		/* Look for next From line */
@@ -880,6 +883,7 @@ int	kill_messages;
 	last_pos = ftell (fp);
 
 	clearerr(fp);
+	free_buffer (mb);
 }
 
 is_new_mail ()
@@ -892,7 +896,7 @@ is_new_mail ()
 
 	fstat (fileno(mail_fp), &s_buf);
 
-	return (s_buf.st_atime < s_buf.st_mtime);
+	return (last_mail_read < s_buf.st_mtime);
 }
 
 /* Locking code doesn't work at the moment - it was intended to prevent
@@ -928,10 +932,16 @@ char	*s;
 	strcpy (last_file, s);
 
 	errno = 0;
-	mail_fp=fopen(s, "r+t");
+	mail_fp = fopen(s, "r+t");
 	if (!mail_fp) {
-		return -1;
+		mail_fp = fopen (s, "rt");
+		if (!mail_fp) {
+			return -1;
+		}
+		read_only = TRUE;
 	}
+	else
+		read_only = FALSE;
 
 #ifdef USE_LOCKING
 #ifdef USE_LOCKF
@@ -1016,6 +1026,7 @@ int	save_all;
 	char	*m_in,*l,*d;
 	int	lines;
 	char	temp [LINE_LENGTH];
+	BUFFER	*body;
 
 	m_in = (char *)m->header->message;
 
@@ -1055,9 +1066,10 @@ int	save_all;
 			}
 	}
 
-	lines = count_lines(m->message->message,m->message->length);
+	body = message_contents (m);
+	lines = count_lines(body->message,body->length);
 
-	fprintf(fp,"Content-Length: %d\n",m->message->length);
+	fprintf(fp,"Content-Length: %d\n",body->length);
 	fprintf(fp,"X-Lines: %d\n",lines);
 
 	switch (m->status) {
@@ -1078,8 +1090,8 @@ int	save_all;
 	if (l)
 		fprintf(fp,"Status: %s\n",l);
 
-	if (m->message && m->message->message) 
-		fprintf(fp,"\n%s\n",m->message->message);
+	if (body && body->message) 
+		fprintf(fp,"\n%s\n",body->message);
 	else
 		fprintf(fp, "\n\n");
 }
@@ -1209,6 +1221,12 @@ int	save_changes ()
 	if (!mail_fp)
 		return;
 
+	if (read_only) {
+		if (read_only_notice_proc ())
+			close_mail_file ();
+		return;
+	}
+
 	/* Create path names */
 
 	sprintf(lock_file,"%s.lock",last_file);
