@@ -1,6 +1,6 @@
 
 /*
- * @(#)main.c	1.17 9/23/94
+ * @(#)main.c	1.19 12/16/94
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -29,6 +29,7 @@
 #include "mailrc.h"
 
 char	default_mail_file[MAXPATHLEN];
+FILE	*mail_fp;
 
 static	LIST	mailrc;
 static	LIST	alias;
@@ -570,6 +571,10 @@ char	*argv[];
 
 	init_pgplib();
 
+	/* Initialise messages.c */
+
+	init_messages();
+
 	s = getenv("MAIL");
 
 	if (s)
@@ -590,6 +595,7 @@ char	*argv[];
 	setup_display(security_level,s);
 
 	close_mail_file ();
+	close_messages ();
 	close_pgplib();
 	exit (0);
 }
