@@ -1,6 +1,6 @@
 
 /*
- *	@(#)messages.c	1.13 9/28/94
+ *	@(#)messages.c	1.18 12/16/94
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -21,6 +21,8 @@
 #include "buffers.h"
 #include "message.h"
 
+extern	FILE	*mail_fp;
+
 void	free_string(s)
 
 char	*s;
@@ -36,10 +38,11 @@ MESSAGE	*new_message()
 
 {
 	MESSAGE	*m;
+	static	word32	id = 1;
 
 	m = (MESSAGE *)malloc(sizeof(MESSAGE));
 
-	m->message = 0;
+	m->body = 0;
 	m->header = 0;
 	m->signature = 0;
 	m->decrypted = 0;
@@ -56,6 +59,9 @@ MESSAGE	*new_message()
 	m->description = 0;
 	m->message_id = 0;
 	m->header_date = 0;
+	m->offset = 0;
+	m->data_type = DT_NONE;
+	m->unique_id = id++;
 
 	m->flags = 0;
 
@@ -67,8 +73,8 @@ free_message(b)
 MESSAGE	*b;
 
 {
-	if (b->message)
-		free_buffer (b->message);
+	if (b->body)
+		free_buffer (b->body);
 	if (b->header)
 		free_buffer (b->header);
 	if (b->signature)
@@ -157,3 +163,87 @@ MESSAGE		*m;
 	if (m->flags & MESS_ENCRYPTED)
 		l->encrypted++;
 }
+
+#define BUF_SIZE	512
+
+static	BUFFER	*messb = NULL;
+
+BUFFER	*message_contents(m)
+
+MESSAGE	*m;
+
+{
+	static	word32 	last_id = 0;
+	byte	buf [BUF_SIZE];
+	int	l, s;
+
+	switch (m->data_type) {
+
+		case DT_MEM:
+		return m->body;
+
+		case DT_FILE:
+
+		/* Is this the same message as last time ? */
+
+		if (m->unique_id == last_id && messb)
+			return messb;
+
+		/* Ok, read it from the file */
+
+		if (!messb)
+			messb = new_buffer ();
+		else
+			clear_buffer (messb);
+
+		fseek (mail_fp, m->offset, 0);
+		l = m->size;
+		while (l > 0) {
+			if (l > BUF_SIZE)
+				s = BUF_SIZE;
+			else
+				s = l;
+
+			fread (buf, s, 1, mail_fp);
+			add_to_buffer (messb, buf, s);
+			l -= s;
+		}
+
+		last_id = m->unique_id;
+
+		bzero (buf, BUF_SIZE);
+		return messb;
+
+		default:
+		return NULL;
+	}
+}
+
+set_mem_message (m)
+
+MESSAGE	*m;
+
+{
+	m->data_type = DT_MEM;
+}
+
+set_file_message (m)
+
+MESSAGE	*m;
+
+{
+	m->data_type = DT_FILE;
+}
+
+init_messages ()
+
+{
+}
+
+close_messages ()
+
+{
+	if (messb)
+		free_buffer (messb);
+}
+
