@@ -1,5 +1,5 @@
 /*
- *	@(#)x.c	1.33 10/23/94
+ *	@(#)x.c	1.36 12/16/94
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -315,6 +315,24 @@ int	f;
 			NULL);
 }
 
+int	read_only_notice_proc ()
+
+{
+	int	choice = NOTICE_YES;
+
+	choice = notice_prompt(top_panel, NULL,
+		NOTICE_MESSAGE_STRINGS, "Mail file is read-only so changes", 
+			"Cannot be saved.",
+			NULL,
+		NOTICE_BUTTON_YES, "Abort",
+		NOTICE_BUTTON_NO, "Ignore",
+		NULL);
+
+	update_random ();
+
+	return (choice == NOTICE_YES);
+}
+
 /* Warn the user that we failed to save correctly */
 
 int	failed_save_notice_proc ()
@@ -879,6 +897,7 @@ static	reply_to_sender()
 	BUFFER	*b;
 	char	subject[256];
 	char	*indent;
+	int	indent_l = 2;
 
 	update_random();
 
@@ -889,6 +908,8 @@ static	reply_to_sender()
 
 	if (!(indent = find_mailrc("indentprefix"))) 
 		indent = "> ";
+	else
+		indent_l = strlen (indent);
 
 	setup_send_window();
 	
@@ -920,7 +941,7 @@ static	reply_to_sender()
 	if (last_message_read->decrypted)
 		b = last_message_read->decrypted;
 	else
-		b = last_message_read->message;
+		b = message_contents(last_message_read);
 
 	mess = b->message;
 	i = b->length;
@@ -934,8 +955,9 @@ static	reply_to_sender()
 		strlen(attribution_string));
 
 	while (i > 0) {
-		textsw_insert(deliver_body_window,
-			indent,2);
+		if (indent_l)
+			textsw_insert(deliver_body_window,
+				indent, indent_l);
 		m = mess;
 		while (i-- && *m && *m!= '\n')
 			m++;
@@ -2141,6 +2163,7 @@ int	level;
 	xv_set (canvas_paint_window(list_canvas), 
 		WIN_EVENT_PROC, list_notify_proc,
 		WIN_CONSUME_EVENTS, WIN_MOUSE_BUTTONS, NULL,
+		WIN_RETAINED, FALSE,
 		NULL);
 
 	newmail_icon_image = (Server_image) xv_create(NULL,SERVER_IMAGE,
